<?php

declare(strict_types=1);

namespace Drupal\lms_messages\Hook;

use Drupal\Core\Hook\Attribute\Hook;
use Drupal\group\Entity\GroupRelationshipInterface;
use Drupal\group\Entity\GroupMembershipInterface;
use Drupal\lms\Entity\CourseStatusInterface;
use Drupal\lms_messages\MessageService;
use Symfony\Component\DependencyInjection\Attribute\Autowire;
use Symfony\Component\DependencyInjection\Attribute\AutowireCallable;

/**
 * Contains message-related hooks.
 */
final class MessageHooks {

  public function __construct(
    #[AutowireCallable(service: 'entity_type.manager', method: 'getStorage', lazy: TRUE)]
    private \Closure $getStorage,
    #[AutowireCallable(service: 'current_user', method: 'id', lazy: TRUE)]
    private \Closure $getCurrentUserId,
    #[Autowire(service: MessageService::class, lazy: TRUE)]
    protected readonly MessageService $messageService,
  ) {}

  #[Hook('lms_course_status_update')]
  public function courseStatusUpdate(CourseStatusInterface $course_status) {
    if ($course_status->isSyncing()) {
      return;
    }

    // Only act if there has been a needs evaluation -> passed or failed or
    // any -> needs evaluation status change.
    $status = $course_status->getStatus();
    if ($status === CourseStatusInterface::STATUS_PROGRESS) {
      return;
    }
    if (!$course_status->original instanceof CourseStatusInterface) {
      return;
    }

    $previous_status = $course_status->original->getStatus();
    if (
      $status === CourseStatusInterface::STATUS_NEEDS_EVALUATION &&
      $previous_status !== CourseStatusInterface::STATUS_NEEDS_EVALUATION
    ) {
      $this->messageService->courseNeedsEvaluation($course_status);
    }
    elseif (
      $previous_status === CourseStatusInterface::STATUS_NEEDS_EVALUATION &&
      \in_array($status, [CourseStatusInterface::STATUS_PASSED, CourseStatusInterface::STATUS_FAILED], TRUE)
    ) {
      $this->messageService->courseEvaluated($course_status);
    }
  }

  #[Hook('group_relationship_insert')]
  public function groupRelationshipInsert(GroupRelationshipInterface $group_relationship) {
    if ($group_relationship->isSyncing()) {
      return;
    }
    if (!$group_relationship instanceof GroupMembershipInterface) {
      return;
    }
    if ($group_relationship->getGroupTypeId() !== 'lms_class') {
      return;
    }

    $this->messageService->studentAddedToClass($group_relationship);
  }

  #[Hook('mail')]
  public function mail(string $key, array &$message, array $params): void {
    $message['subject'] = $params['mail_subject'];
    $message['body'][] = $params['mail_body'];
  }

}
