<?php

namespace Drupal\lms_messages;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\lms\Entity\CourseStatusInterface;
use Drupal\group\Entity\GroupMembershipInterface;
use Symfony\Component\DependencyInjection\Attribute\AutowireCallable;

/**
 * General message related logic.
 */
class MessageService {

  public const CONFIG_NAME = 'lms_messages.settings';

  public function __construct(
    #[AutowireCallable(service: EntityTypeManagerInterface::class, method: 'getStorage')]
    private \Closure $getStorage,
    #[AutowireCallable(service: AccountInterface::class, method: 'id', lazy: TRUE)]
    private \Closure $getCurrentUserId,
    #[AutowireCallable(service: ConfigFactoryInterface::class, method: 'get', lazy: TRUE)]
    private \Closure $getConfig,
    #[AutowireCallable(service: 'message_notify.sender', method: 'send', lazy: TRUE)]
    private \Closure $sendMessage,
  ) {}

  /**
   * Course has just been evaluated, create a message for the student.
   */
  public function courseEvaluated(CourseStatusInterface $course_status): void {
    $template = ($this->getStorage)('message_template')->load('course_evaluated');
    if ($template === NULL || !$template->status()) {
      return;
    }

    $this->createMessage([
      'template' => $template->id(),
      'uid' => $course_status->getUserId(),
      'course' => $course_status->getCourseId(),
    ]);
  }

  /**
   * Course needs evaluating, create a message for the teacher(s).
   */
  public function courseNeedsEvaluation(CourseStatusInterface $course_status): void {
    $template = ($this->getStorage)('message_template')->load('course_needs_evaluation');
    if ($template === NULL || !$template->status()) {
      return;
    }

    $course = $course_status->getCourse();
    foreach ($course->getMembers() as $member) {
      $account = $member->getUser();
      if ($account->isAnonymous()) {
        continue;
      }

      $this->createMessage([
        'template' => $template->id(),
        'uid' => $account->id(),
        'course' => $course_status->getCourseId(),
        'user' => $course_status->getUserId(),
      ]);
    }
  }

  /**
   * Student has been added to a class.
   */
  public function studentAddedToClass(GroupMembershipInterface $group_membership) {
    $template = ($this->getStorage)('message_template')->load('student_added_to_class');
    if ($template === NULL || !$template->status()) {
      return;
    }

    $account = $group_membership->getEntity();
    if ((int) $account->id() === (int) ($this->getCurrentUserId)()) {
      return;
    }

    $this->createMessage([
      'template' => $template->id(),
      'uid' => $account->id(),
      'class' => $group_membership->getGroupId(),
    ]);
  }

  /**
   * Create a message helper.
   */
  private function createMessage(array $properties): void {
    $message = (($this->getStorage)('message'))->create($properties);

    $template_behavior = ($this->getConfig)(self::CONFIG_NAME)->get('template_behavior');
    if (!\is_array($template_behavior)) {
      $message->save();
      return;
    }

    $template_id = $message->getTemplate()->id();
    if (!\array_key_exists($template_id, $template_behavior)) {
      $message->save();
      return;
    }

    // Send an email if configured.
    if ($template_behavior[$template_id]['email'] === TRUE) {
      ($this->sendMessage)($message);
    }

    // Save message if not specified otherwise.
    if ($template_behavior[$template_id]['prevent_message_save'] !== TRUE) {
      $message->save();
    }
  }

}
